/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/unitcl.h,v 1.14 2004/07/08 13:33:43 brandt Exp $
 */
#ifndef unitcl_h_
#define unitcl_h_

/*
 * forward declarations
 */
struct uni_iehdr;
struct unicx;
struct uni_addr;
struct uni_subaddr;
struct uni_sap;
struct unisve_blli;
struct unisve_blli_id2;
struct unisve_blli_id3;
struct unisve_bhli;
struct unisve_addr;
struct unisve_selector;
struct uni_cref;
struct uni_msghdr;
struct uni_all;
union uni_msgall;
union uni_ieall;
struct atm_resp;
struct atm_port_info;
struct atm_exstatus;
struct atm_exstatus_ep;
struct atm_exstatus_port;
struct atm_exstatus_conn;
struct atm_exstatus_party;

#include "unitcl_iefunc.h"
#include "unitcl_msgfunc.h"

int fmt_msg(Tcl_Interp *, Tcl_DString *, const struct uni_all *);
int parse_msg(Tcl_Interp *, int, const char **, struct uni_all *,
	struct unicx *);

int fmt_mtype(Tcl_Interp *, Tcl_DString *, u_int mtype);
int parse_mtype(Tcl_Interp *, const char *, u_int *mtype);

int fmt_msgact(Tcl_Interp *, Tcl_DString *, u_int msgact);
int parse_msgact(Tcl_Interp *, const char *, struct uni_msghdr *);

int fmt_msghdr(Tcl_Interp *, Tcl_DString *, u_int mtype,
	const struct uni_msghdr *);
int parse_msghdr(Tcl_Interp *, int, const char **,
	u_int *mtype, struct uni_msghdr *h);

void fmt_cref(Tcl_DString *, const struct uni_cref *);
int parse_cref(Tcl_Interp *, int, const char **, struct uni_cref *);

int parse_ie(Tcl_Interp *, const char *, union uni_ieall *, u_int *ietype);

/* SAPs */
int parse_svetag(Tcl_Interp *, const char *, u_int *tag);
int fmt_svetag(Tcl_Interp *, Tcl_DString *, u_int);

int parse_addr_sve(Tcl_Interp *, const char *, struct unisve_addr *);
int fmt_addr_sve(Tcl_Interp *, Tcl_DString *, const struct unisve_addr *);

int parse_selector_sve(Tcl_Interp *, const char *, struct unisve_selector *);
int fmt_selector_sve(Tcl_Interp *, Tcl_DString *,
	const struct unisve_selector *);

int parse_blli_id2_sve(Tcl_Interp *, const char *, struct unisve_blli_id2 *);
int fmt_blli_id2_sve(Tcl_Interp *, Tcl_DString *,
	const struct unisve_blli_id2 *);

int parse_blli_id3_sve(Tcl_Interp *, const char *, struct unisve_blli_id3 *);
int fmt_blli_id3_sve(Tcl_Interp *, Tcl_DString *,
	const struct unisve_blli_id3 *);

int parse_bhli_sve(Tcl_Interp *, const char *, struct unisve_bhli *);
int fmt_bhli_sve(Tcl_Interp *, Tcl_DString *, const struct unisve_bhli *);

int parse_sap(Tcl_Interp *, int, const char **, struct uni_sap *);
int fmt_sap(Tcl_Interp *, Tcl_DString *, const struct uni_sap *);

int fmt_addr(Tcl_Interp *, Tcl_DString *, const struct uni_addr *);
int fmt_subaddr(Tcl_Interp *, Tcl_DString *, const struct uni_subaddr *);
int parse_addr(Tcl_Interp *, const char **, struct uni_addr *);
int parse_subaddr(Tcl_Interp *, const char **, struct uni_subaddr *);

char * substitute_vars(Tcl_Interp *, const char *);

union uniapi_all;

int fmt_uniapi_sig(Tcl_Interp *, Tcl_DString *str, u_int sig);
int fmt_uniapi(Tcl_Interp *interp, Tcl_DString *str, u_int sig,
	const void *ptr, size_t);

int parse_uniapi_sig(Tcl_Interp *, const char *, u_int *sig);
int parse_uniapi(Tcl_Interp *interp, int argc, const char **argv, u_int sig,
	union uniapi_all *u, size_t *, struct unicx *);

int unitcl_setres(Tcl_Interp *, const char *, ...);

int fmt_sscop(Tcl_Interp *, Tcl_DString *, const struct uni_msg *);
int parse_sscop(Tcl_Interp *, struct uni_msg *, int, const char **);

int fmt_sscop_sig(Tcl_Interp *, Tcl_DString *, u_int sig, u_int arg,
    const struct uni_msg *);
int parse_sscop_sig(Tcl_Interp *, u_int *sig, u_int *arg, struct uni_msg **,
    int, const char **);

int fmt_sscop_msig(Tcl_Interp *, Tcl_DString *, u_int sig, u_int arg1,
    u_int arg2, const struct uni_msg *);
int parse_sscop_msig(Tcl_Interp *, u_int *sig, u_int *arg1, u_int *arg2,
    struct uni_msg **, int, const char **);

int fmt_saal_sig(Tcl_Interp *, Tcl_DString *, u_int, const struct uni_msg *);
int parse_saal_sig(Tcl_Interp *, u_int *, int, const char *[],
    struct uni_msg *);

/* Get the name of an API signal */
int fmt_atmapi_sig(Tcl_Interp *, Tcl_DString *, u_int);

/* Parse API signal name */
int parse_atmapi_sig(Tcl_Interp *, const char *, u_int *);

/* Format an API signal */
int fmt_atmapi(Tcl_Interp *, Tcl_DString *, u_int, const u_char *, size_t);

/* Parse ATMAPI signal */
int parse_atmapi(Tcl_Interp *, int , const char **, u_int, u_char *,
    size_t *, struct unicx *);

/* Format/parse local port info */
int fmt_port_info(Tcl_DString *, const struct atm_port_info *);
int parse_port_info(Tcl_Interp *, int, const char **, struct atm_port_info *);

/* Format extended CC status */
int atmapi_fmt_exstatus(Tcl_Interp *, Tcl_DString *,
    const struct atm_exstatus *, struct atm_exstatus_ep *,
    struct atm_exstatus_port *, struct atm_exstatus_conn *,
    struct atm_exstatus_party *);

#endif
